/*
 * Chapter 24
 *
 * Temperature is measured with NTC on PA1 pin (ADC1).
 * The value of the AD converter is first converted to voltage and then to temperature.
 * We receive a number via UART to set a temperature threshold.
 * The threshold is used to determine when the fan, connected to PD7 pin, should turn on.
 * The fan can be controlled via PWM, but for now only on / off.
 * 
 * Homework: supplement the code to gradually change the PWM duty cycle between the two boundaries.
 */

#include "../Headers/main.h"

volatile uint32_t cntr = 0;
volatile int32_t U_NTC = 0;
volatile int32_t temperature = 0;
volatile uint8_t adc_value = 0;
volatile uint8_t limit = 25;

ISR(TIMER1_OVF_vect)
{
    // The counter is increased, if it hasn't reached the maximum yet
    if (cntr < CNTR_MAX)
    {
        cntr++;
    }

    // If it has reached
    else
    {
        cntr = 0;
        // AD conversion is started (sampling)
        ADCStart();
    }
}

// AD conversion is completed, AD interrupt
ISR(ADC_vect)
{
    // The value read from the ADC register is converted to voltage, and saved
    adc_value = ADCH;
}

// Data is incoming via UART port
ISR(USART_RXC_vect)
{
	// Saving data into variable
	limit=UDR;
}

int main(void)
{
    IOInit();
    TimerInit();
    PWMInit();
    ADCInit();
	
	//Enabling AD
	ADCEnable();
	
	// Initialization of UART, 103=4800 baud, Page 165 of ATmega16A datasheet
	UARTInit(103);

	FILE uart_output = FDEV_SETUP_STREAM(UARTPutchar,	NULL,			_FDEV_SETUP_WRITE);
	FILE uart_input  = FDEV_SETUP_STREAM(NULL,			UARTGetchar,	_FDEV_SETUP_READ);
	stdout = &uart_output;
	stdin  = &uart_input;
	
	// Testing
	puts("Program elindult!");

    //Infinite loop
    while (1)
    {
		// Converting the ADC value into voltage
		U_NTC = adc_value;
		U_NTC = U_NTC * ADC_CONST;
		
		// Converting the voltage value into temperature value
		// this linear approximation gives a correct value roughly between -10 and 40C
		// for the time being, please accept this approximation
		temperature = (-172 * U_NTC + 602530)/10000;
		
		// If the measured temperature is higher, than the limit set on UART
		if (temperature > limit)
		{
			// Turning on the fan (duty cycle 100%)
			OCR2 = 255;
		}
		else
		{
			// Turning off the fan (duty cycle 0%)
			OCR2 = 0;
		}
		// Sending of measured voltage and temperature, and the set limit
		printf("mert: %ld mV     %ld C    limit: %d C\n", U_NTC, temperature, limit);
		_delay_ms(1000);
    }

    return 0;
}